package com.bsz.school_send_back_end.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Businessinfo;
import com.bsz.school_send_back_end.model.dto.CommonRequest;
import com.bsz.school_send_back_end.model.dto.businessinfo.InfoAddRequest;
import com.bsz.school_send_back_end.model.dto.businessinfo.InfoQueryRequest;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.BusinessinfoService;
import com.github.houbb.sensitive.word.core.SensitiveWordHelper;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;

@Tag(name = "商家公告接口")
@RestController
@Slf4j
@RequestMapping("/businessInfo")
public class BusinessInfoController {

    @Resource
    private BusinessinfoService businessinfoService;

    @Resource
    private BusinessService businessService;

    /**
     * 添加公告
     * @param addRequest 添加请求体
     * @param request 网络请求
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<Boolean> addInfo (@RequestBody InfoAddRequest addRequest, HttpServletRequest request) {
        if ( addRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Business loginBusiness = businessService.getLoginBusiness(request);
        Businessinfo businessinfo = new Businessinfo();
        String content = addRequest.getContent();
        boolean contains = SensitiveWordHelper.contains(content);
        ThrowUtils.throwIf(contains, ErrorCode.PARAMS_ERROR, "公告含有违禁词");
        BeanUtils.copyProperties(addRequest, businessinfo);
        businessinfo.setBusinessId(loginBusiness.getId());
        boolean save = businessinfoService.save(businessinfo);
        ThrowUtils.throwIf(!save, ErrorCode.SYSTEM_ERROR, "添加公告失败");
        return ResultUtils.success(true);
    }

    /**
     * 删除公告
     * @param commonRequest 删除请求体
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<Boolean> deleteInfo (@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        Long id = commonRequest.getId();
        LambdaQueryWrapper<Businessinfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Businessinfo::getId, id);
        queryWrapper.eq(Businessinfo::getBusinessId, businessId);
        long count = businessinfoService.count(queryWrapper);
        ThrowUtils.throwIf(count != 1, ErrorCode.SYSTEM_ERROR, "当前公告未找到");
        boolean remove = businessinfoService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.SYSTEM_ERROR, "删除失败");
        return ResultUtils.success(true);
    }

    /**
     * 查询公告列表 （仅商家）
     * @param request 网络请求
     * @return 查询列表
     */
    @PostMapping("/list/my")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<List<Businessinfo>> selectMyInfo (HttpServletRequest request) {
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        LambdaQueryWrapper<Businessinfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Businessinfo::getBusinessId, businessId);
        queryWrapper.orderByDesc(Businessinfo::getCreateTime);
        List<Businessinfo> list = businessinfoService.list(queryWrapper);

        return ResultUtils.success(list, "查询成功");
    }

    /**
     * 查询公列表（仅管理员， 分页）
     * @param infoQueryRequest 查询请求体
     * @return 查询列表
     */
    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Businessinfo>> selectInfoByPage (@RequestBody InfoQueryRequest infoQueryRequest) {
        if (infoQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = infoQueryRequest.getCurrent();
        long pageSize = infoQueryRequest.getPageSize();

        QueryWrapper<Businessinfo> queryWrapper = businessinfoService.getQueryWrapper(infoQueryRequest);
        Page<Businessinfo> page = businessinfoService.page(new Page<>(current, pageSize), queryWrapper);
        return ResultUtils.success(page, "查询成功");
    }


    /**
     * 用户查询商家最新公告
     * @return 查询列表
     */
    @PostMapping("/list")
    public BaseResponse<Businessinfo> selectInfo (@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long businessId = commonRequest.getId();
        LambdaQueryWrapper<Businessinfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Businessinfo::getBusinessId, businessId);
        queryWrapper.orderByDesc(Businessinfo::getCreateTime);
        queryWrapper.last("LIMIT 1");
        Businessinfo businessinfo = businessinfoService.getOne(queryWrapper);
        return ResultUtils.success(businessinfo, "查询成功");
    }
}
