package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Category;
import com.bsz.school_send_back_end.model.dto.category.CategoryAddRequest;
import com.bsz.school_send_back_end.model.dto.category.CategoryDeleteRequest;
import com.bsz.school_send_back_end.model.dto.category.CategoryUpdateRequest;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.CategoryService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.Resource;
import java.util.List;

@Tag(name = "餐厅接口")
@Slf4j
@RestController
@RequestMapping("/category")
public class CategoryController {

    @Resource
    private CategoryService categoryService;

    @Resource
    private BusinessService businessService;

    /**
     * 添加类别
     * @param categoryAddRequest 添加请求体
     * @return 是否成功
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Long> addCategory (@RequestBody CategoryAddRequest categoryAddRequest) {
        if (categoryAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Category category = new Category();
        BeanUtils.copyProperties(categoryAddRequest,  category);
        categoryService.validCategory(category, true);
        boolean save = categoryService.save(category);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        Long id = category.getId();
        return ResultUtils.success(id,"添加成功");
    }

    /**
     * 修改类别
     * @param categoryUpdateRequest 修改请求体
     * @return 返回修改后类别
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Category> updateCategory (@RequestBody CategoryUpdateRequest categoryUpdateRequest) {
        if (categoryUpdateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Category category = new Category();
        BeanUtils.copyProperties(categoryUpdateRequest, category);
        categoryService.validCategory(category, false);
        boolean update = categoryService.updateById(category);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "操作失败");

        return new BaseResponse<>(0,category,"修改成功");
    }

    /**
     * 根据id删除
     * @param categoryDeleteRequest 删除请求体
     * @return 是否成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCategory (@RequestBody CategoryDeleteRequest categoryDeleteRequest) {
        if (categoryDeleteRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        LambdaQueryWrapper<Business> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Business::getCategoryId, categoryDeleteRequest.getId());
        long count = businessService.count(wrapper);
        ThrowUtils.throwIf(count > 0, ErrorCode.OPERATION_ERROR, "该分类下拥有商家");
        boolean remove = categoryService.removeById(categoryDeleteRequest.getId());
        return ResultUtils.success(remove);
    }

    /**
     * 根据id查询分类
     */
    @GetMapping("/getById")
    public BaseResponse<Category> getCategoryById (long id) {
        if (id <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Category category = categoryService.getById(id);
        ThrowUtils.throwIf(category == null, ErrorCode.NOT_FOUND_ERROR);
        return ResultUtils.success(category);
    }

    /**
     * 查询类别
     */
    @GetMapping("/list")
    public BaseResponse<List<Category>> listCategoryAll() {
        List<Category> categoryList = categoryService.list();
        return ResultUtils.success(categoryList);
    }
}
