package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.annotation.SystemLog;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.SystemLogConstant;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Dishes;
import com.bsz.school_send_back_end.model.dto.Dishes.DishesAddRequest;
import com.bsz.school_send_back_end.model.dto.Dishes.DishesDeleteRequest;
import com.bsz.school_send_back_end.model.dto.Dishes.DishesQueryRequest;
import com.bsz.school_send_back_end.model.dto.Dishes.DishesUpdateRequest;
import com.bsz.school_send_back_end.model.vo.DishesVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.DishesService;
import com.bsz.school_send_back_end.service.SpecificationsDishesService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;

@Tag(name = "菜品接口")
@Slf4j
@RestController
@RequestMapping("/dishes")
public class DishesController {
    @Resource
    private DishesService dishesService;

    @Resource
    private BusinessService businessService;

    @Resource
    private SpecificationsDishesService specificationsDishesService;

    /**
     *添加菜品
     * @param dishesAddRequest 添加菜品请求体
     * @return 是否成功
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(executeStep = SystemLogConstant.ADD_DISHES)
    public BaseResponse<Boolean> addDishes (@RequestBody DishesAddRequest dishesAddRequest, HttpServletRequest request) {
        if (dishesAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Dishes dishes = new Dishes();
        BeanUtils.copyProperties(dishesAddRequest,dishes);
        //校验
        dishesService.validDishes(dishes,false);
        //添加商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        dishes.setBusinessId(businessId);
        //保存菜品信息
        Long dishesId = dishesService.addDishes(dishes);
        //中间表中添加数据
        List<Long> specificationsIds = dishesAddRequest.getSpecificationsIds();
        specificationsDishesService.add(dishesId,specificationsIds);
        return ResultUtils.success(true);
    }

    /**
     * 删除菜品
     * @param dishesDeleteRequest 删除请求
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(executeStep = SystemLogConstant.DELETE_DISHES)
    public BaseResponse<Boolean> deleteDishes(@RequestBody DishesDeleteRequest dishesDeleteRequest,HttpServletRequest request) {
        if (dishesDeleteRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long dishesId = dishesDeleteRequest.getId();
        //获取商家id
        Business business = businessService.getLoginBusiness(request);
        Long businessId = business.getId();
        //校验菜品是不是当前登录商家的
        LambdaQueryWrapper<Dishes> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Dishes::getId, dishesId);
        wrapper.eq(Dishes::getBusinessId, businessId);
        Dishes dishes = dishesService.getOne(wrapper);
        ThrowUtils.throwIf(dishes == null, ErrorCode.OPERATION_ERROR, "当前商家无权限删除该菜品");
        //删除菜品
        boolean dishesRemove = dishesService.removeById(dishesId);
        ThrowUtils.throwIf(!dishesRemove, ErrorCode.OPERATION_ERROR);
        //删除中间表信息
        specificationsDishesService.deleteByDishesId(dishesId);
        return ResultUtils.success(true);
    }

    /**
     * 更新菜品
     * @param dishesUpdateRequest 更新请求
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @Transactional(rollbackFor = Exception.class)
    @SystemLog(executeStep = SystemLogConstant.UPDATE_DISHES)
    public BaseResponse<Boolean> updateDishes (@RequestBody DishesUpdateRequest dishesUpdateRequest, HttpServletRequest request) {
        if (dishesUpdateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Dishes dishes = new Dishes();
        BeanUtils.copyProperties(dishesUpdateRequest, dishes);
        //校验
        dishesService.validDishes(dishes, true);
        //添加商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        dishes.setBusinessId(businessId);
        //更新菜品
        Boolean updateDishes = dishesService.updateDishes(dishes);
        ThrowUtils.throwIf(!updateDishes, ErrorCode.OPERATION_ERROR);
        //更新中间表的信息
        Long dishesId = dishes.getId();
        List<Long> specificationsIds = dishesUpdateRequest.getSpecificationsIds();
        return ResultUtils.success(specificationsDishesService.updateByDishesId(dishesId, specificationsIds));
    }

    /**
     * 更新菜品状态
     * @param dishesUpdateRequest 菜品请求体
     * @param request 前端请求
     * @return 是否成功
     */
    @PostMapping("/update/status")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @SystemLog(executeStep = SystemLogConstant.UPDATE_DISHES)
    public BaseResponse<Boolean> updateDishesStatus (@RequestBody DishesUpdateRequest dishesUpdateRequest, HttpServletRequest request) {
        Long id = dishesUpdateRequest.getId();
        String status = dishesUpdateRequest.getStatus();
        if (id == null || StringUtils.isBlank(status)) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "请补全参数");
        }
        //获取商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        //更新状态
        LambdaUpdateWrapper<Dishes> wrapper = new LambdaUpdateWrapper<>();
        wrapper.eq(Dishes::getId, id);
        wrapper.eq(Dishes::getBusinessId, businessId);
        wrapper.set(Dishes::getStatus, status);
        return ResultUtils.success(dishesService.update(wrapper));
    }

    /**
     *获取菜品脱敏列表
     * @param dishesQueryRequest 查询请求体
     * @return 脱敏列表
     */
    @PostMapping("/list/dishes")
    public BaseResponse<List<Dishes>> dishesList(@RequestBody DishesQueryRequest dishesQueryRequest) {
        Long businessId = dishesQueryRequest.getBusinessId();
        if (businessId == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        //获取查询条件
        QueryWrapper<Dishes> wrapper = dishesService.getWrapper(dishesQueryRequest);
        //根据查询条件该商家的所有菜品
        List<Dishes> list = dishesService.list(wrapper);
        //获取菜品脱敏列表
        return ResultUtils.success(list);
    }

    /**
     *获取菜品脱敏列表
     * @param dishesQueryRequest 查询请求体
     * @return 分页列表
     */
    @PostMapping("/list/page/dishesVO")
    public BaseResponse<Page<DishesVO>> dishesVOPage(@RequestBody DishesQueryRequest dishesQueryRequest) {
        long current = dishesQueryRequest.getCurrent();
        long pageSize = dishesQueryRequest.getPageSize();
        //获取查询条件
        QueryWrapper<Dishes> wrapper = dishesService.getWrapper(dishesQueryRequest);
        //获取商家多有菜品
        Page<Dishes> dishesPage = dishesService.page(new Page<>(current, pageSize), wrapper);
        //获取脱敏信息列表
        List<Dishes> dishesList = dishesPage.getRecords();
        long total = dishesPage.getTotal();
        List<DishesVO> dishesVOList = dishesService.getDishesVOList(dishesList);
        Page<DishesVO> dishesVOPage = new Page<>(current, pageSize, total);
        dishesVOPage.setRecords(dishesVOList);
        return ResultUtils.success(dishesVOPage);
    }
}
