package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.annotation.SystemLog;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.SystemLogConstant;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Dishes;
import com.bsz.school_send_back_end.model.domain.DishesGroup;
import com.bsz.school_send_back_end.model.dto.DishesGroup.DishesGroupAddRequest;
import com.bsz.school_send_back_end.model.dto.DishesGroup.DishesGroupDeleteRequest;
import com.bsz.school_send_back_end.model.dto.DishesGroup.DishesGroupQueryRequest;
import com.bsz.school_send_back_end.model.dto.DishesGroup.DishesGroupUpdateRequest;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.DishesGroupService;
import com.bsz.school_send_back_end.service.DishesService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;

@Tag(name = "菜品分组接口")
@Slf4j
@RestController
@RequestMapping("/dishesGroup")
public class DishesGroupController {
    @Resource
    private BusinessService businessService;

    @Resource
    private DishesGroupService dishesGroupService;

    @Resource
    private DishesService dishesService;

    /**
     * 添加菜品分组
     * @param dishesGroupAddRequest 添加请求
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @SystemLog(executeStep = SystemLogConstant.ADD_DISHES_GROUP)
    public BaseResponse<Boolean> addDishesGroup(@RequestBody DishesGroupAddRequest dishesGroupAddRequest, HttpServletRequest request) {
        if (dishesGroupAddRequest == null) {
            throw new  BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }

        DishesGroup dishesGroup = new DishesGroup();
        BeanUtils.copyProperties(dishesGroupAddRequest,dishesGroup);
        //校验请求体
        dishesGroupService.validDishesGroup(dishesGroup,false);
        //添加分组
        Boolean result = dishesGroupService.addDishesGroup(dishesGroup, request);
        ThrowUtils.throwIf(!result,ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }

    /**
     * 删除菜品分组
     * @param dishesGroupDeleteRequest 删除请求
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/delete")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @SystemLog(executeStep = SystemLogConstant.DELETE_DISHES_GROUP)
    public BaseResponse<Boolean> deleteDishesGroup (@RequestBody DishesGroupDeleteRequest dishesGroupDeleteRequest, HttpServletRequest request) {
        if (dishesGroupDeleteRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = dishesGroupDeleteRequest.getId();
        //获取商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();

        // 校验分组是否为当前商家的分组
        QueryWrapper<DishesGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id",id);
        queryWrapper.eq("businessId",businessId);
        DishesGroup dishesGroup = dishesGroupService.getOne(queryWrapper);
        ThrowUtils.throwIf(dishesGroup == null,ErrorCode.OPERATION_ERROR,"当前商家无权删除该分组");
        //删除分组
        boolean byId = dishesGroupService.removeById(id);
        ThrowUtils.throwIf(!byId,ErrorCode.OPERATION_ERROR);
        //检查分组下是否有菜品
        LambdaQueryWrapper<Dishes> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Dishes::getDishesGroupId, id);
        long count = dishesService.count(wrapper);
        ThrowUtils.throwIf(count > 0, ErrorCode.OPERATION_ERROR, "该分组下还有菜品");
        //删除该分组的菜品以及中检表
        dishesService.removeByDishesGroupId(id);
        return ResultUtils.success(true);
    }

    /**
     * 更行菜品分组
     * @param dishesGroupUpdateRequest 更新请求
     * @param request 网轮廓请求
     * @return 是否成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @SystemLog(executeStep = SystemLogConstant.UPDATE_DISHES_GROUP)
    public BaseResponse<Boolean> updateDishesGroup (@RequestBody DishesGroupUpdateRequest dishesGroupUpdateRequest,HttpServletRequest  request){
        if (dishesGroupUpdateRequest == null) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR);
        }
        DishesGroup dishesGroup = new DishesGroup();
        BeanUtils.copyProperties(dishesGroupUpdateRequest,dishesGroup);
        //校验
        dishesGroupService.validDishesGroup(dishesGroup,true);
        //更新菜品分组信息
        Boolean result = dishesGroupService.updateDishesGroupById(dishesGroup, request);
        ThrowUtils.throwIf(!result,ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 查询菜品分组脱敏列表
     * @param dishesGroupQueryRequest  菜品分组请求体
     * @return 脱敏列表
     */
    @PostMapping("/list/dishesGroup")
    public BaseResponse<List<DishesGroup>> dishesGroupList (@RequestBody DishesGroupQueryRequest dishesGroupQueryRequest){
        //获取商家id
        Long businessId = dishesGroupQueryRequest.getBusinessId();
        Business business = businessService.getById(businessId);
        if (business == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"该商家不存在");
        }
        //查询商家菜品分组信息
        QueryWrapper<DishesGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("businessId",businessId);
        List<DishesGroup> list = dishesGroupService.list(queryWrapper);
        // 获取菜品分组脱敏列表
        return ResultUtils.success(list);
    }
}
