package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.*;
import com.bsz.school_send_back_end.model.dto.CommonRequest;
import com.bsz.school_send_back_end.model.dto.errand.ErrandAddRequest;
import com.bsz.school_send_back_end.model.dto.errand.ErrandQueryRequest;
import com.bsz.school_send_back_end.model.dto.errand.ErrandStateRequest;
import com.bsz.school_send_back_end.model.dto.errand.ErrandUpdateRequest;
import com.bsz.school_send_back_end.model.vo.ErrandVO;
import com.bsz.school_send_back_end.model.vo.OrdersVO;
import com.bsz.school_send_back_end.service.*;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static com.bsz.school_send_back_end.contant.UserConstant.*;

@Tag(name = "快送员接口")
@Slf4j
@RestController
@RequestMapping("/errand")
public class ErrandController {
    @Resource
    private ErrandService errandService;

    @Resource
    private UserService userService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private ErrandAuthService errandAuthService;

    @Resource
    private ErrandOrderService errandOrderService;

    /**
     * 注册跑腿
     * @param addRequest 注册请求体
     * @return 是否成功
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addErrand(@RequestBody ErrandAddRequest addRequest) {
        if (addRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User user = new User();
        Errand errand = new Errand();
        BeanUtils.copyProperties(addRequest, user);
        System.out.println("User account: " + user.getUserAccount());
        BeanUtils.copyProperties(addRequest, errand);

        userService.validUser(user, true);
        errandService.validErrand(errand, true);

        //加密
        String encryptPassword = DigestUtils.md5DigestAsHex((USER_SALT + user.getUserPassword()).getBytes());
        user.setUserPassword(encryptPassword);

        //更改用户权限为跑腿
        user.setUserRole("errand");

        //往user表中补充跑腿信息
        user.setUsername(errand.getErrandName());
        user.setPhone(errand.getErrandPhone());

        errandService.addErrand(user, errand);
        return ResultUtils.success(true);
    }

    /**
     * 删除跑腿信息
     * @param request 当前登录用户
     * @return 是否成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> deleteErrand (HttpServletRequest request){
        errandService.deleteErrand(request);
        return ResultUtils.success(true);
    }

    /**
     * 修改跑腿信息(管理员)
     * @param updateRequest 修改请求体
     * @return 是否修改成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateErrand(@RequestBody ErrandUpdateRequest updateRequest) {
        if (updateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Boolean update = errandService.updateErrand(updateRequest);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR,"更新跑腿失败");

        return ResultUtils.success(true);
    }

    /**
     * 修改跑腿信息（个人）
     * @param updateRequest 修改请求体
     * @param request 当前登录用户
     * @return 是否修改成功
     */
    @PostMapping("/update/my")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    public BaseResponse<Boolean> updateErrandByUser (@RequestBody ErrandUpdateRequest updateRequest, HttpServletRequest request ) {
        if (updateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Errand loginErrand = errandService.getLoginErrand(request);
        
        Integer totalOrders = loginErrand.getTotalOrders();
        ThrowUtils.throwIf(totalOrders > 0, ErrorCode.OPERATION_ERROR, "当前跑腿还有订单未送");
        Boolean update = errandService.updateErrand(updateRequest);
        Long errandId = loginErrand.getId();
        Long id = updateRequest.getId();
        ThrowUtils.throwIf(!id.equals(errandId), ErrorCode.PARAMS_ERROR, "修改的跑腿信息不是当前跑腿");
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "更新跑腿失败");
        return ResultUtils.success(true);
    }

    /**
     * 获取当前跑腿信息列表
     * @return 跑腿信息列表
     */
    @PostMapping("/list")
    @AuthCheck(mustRole = ADMIN_ROLE)
    public BaseResponse<List<Errand>> listErrand() {
        List<Errand> list = errandService.list();
        return ResultUtils.success(list);
    }

    /**
     * 分页获取跑腿信息列表
     * @param queryRequest 查询请求体
     * @return 跑腿信息（分页）
     */
    @PostMapping("/page")
    @AuthCheck(mustRole = ADMIN_ROLE)
    public BaseResponse<Page<Errand>> listErrandByPage (@RequestBody ErrandQueryRequest queryRequest) {
        if (queryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        long pageSize = queryRequest.getPageSize();
        long current = queryRequest.getCurrent();
        QueryWrapper<Errand> queryWrapper = errandService.getQueryWrapper(queryRequest);
        Page<Errand> errandPage = errandService.page(new Page<>(current, pageSize), queryWrapper);
        return ResultUtils.success(errandPage);
    }

    /**
     * 抢单
     * @param commonRequest 订单id
     * @param request 当前登录用户
     * @return 是否抢单成功
     */
    @PostMapping("/get/order")
    @AuthCheck(mustRole = UserConstant.ERRAND_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> getOrder(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long ordersId = commonRequest.getId();
        Errand loginErrand = errandService.getLoginErrand(request);
        Long loginErrandId = loginErrand.getId();
        ErrandOrder errandOrder = errandOrderService.getOne(Wrappers.<ErrandOrder>lambdaQuery()
                .eq(ErrandOrder::getOrderId, ordersId));
        Integer state = loginErrand.getState();
        if (state != 1) {
            throw new BusinessException(ErrorCode.FORBIDDEN_ERROR, "当前跑腿不可抢单");
        }

        errandService.updateTotal(loginErrand, true);

        // 开始事务
        Orders orders = ordersService.getOrderWithPessimisticLock(ordersId);

        // 判断订单是否已被抢
        if (orders == null || orders.getErrandId() != null) {
            return ResultUtils.success(false); // 订单已经被抢或不存在
        }

        // 更新订单的 errandId 和状态
        orders.setErrandId(loginErrandId);
        boolean update = ordersService.updateById(orders);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "抢单失败");


        errandOrder.setErrandId(loginErrandId);
        errandOrder.setErrandState(2); // 2待取货
        boolean update1 = errandOrderService.updateById(errandOrder);
        ThrowUtils.throwIf(!update1, ErrorCode.OPERATION_ERROR, "更新跑腿订单关联失败");

        return ResultUtils.success(true);
    }

    /**
     * 修改订单
     * @param errandStateRequest 订单id
     * @param request 当前登录用户
     * @return 是否修改成功
     */
    @PostMapping("/update/state")
    @AuthCheck(mustRole = ERRAND_ROLE)
    public BaseResponse<Boolean> updateOrdersStateByErrand (@RequestBody ErrandStateRequest errandStateRequest, HttpServletRequest request) {
        if (errandStateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        errandService.orderToErrand(errandStateRequest, request);
        return ResultUtils.success(true);
    }

    /**
     * 根据当前登录用户获取跑腿信息
     */
    @PostMapping("/get/current")
    @AuthCheck(mustRole = ERRAND_ROLE)
    public BaseResponse<ErrandVO> getByErrand (HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);
        ErrandVO errandVO = errandService.getErrandVO(loginErrand);

        return ResultUtils.success(errandVO);
    }

    /**
     * 当前跑腿获取订单
     */
    @PostMapping("/list/order")
    @AuthCheck(mustRole = ERRAND_ROLE)
    public BaseResponse<List<OrdersVO>> listOrdersByErrand(Integer errandState, HttpServletRequest request) {
        Errand loginErrand = errandService.getLoginErrand(request);
        String distributionScope = loginErrand.getDistributionScope();

        // 创建 errand_order 表的查询条件
        LambdaQueryWrapper<ErrandOrder> errandOrderQueryWrapper = new LambdaQueryWrapper<>();
        errandOrderQueryWrapper.eq(ErrandOrder::getErrandState, errandState);

        // 根据 errandState 的不同情况调整查询
        if (errandState != 1) {
            errandOrderQueryWrapper.eq(ErrandOrder::getErrandId, loginErrand.getId());
        } else {
            errandOrderQueryWrapper.isNull(ErrandOrder::getErrandId);
            errandOrderQueryWrapper.ge(ErrandOrder::getEndTime, new Date());
        }

        // 查询符合条件的 errand_order 数据
        List<ErrandOrder> errandOrderList = errandOrderService.list(errandOrderQueryWrapper);

        // 获取符合条件的 orderId 列表
        List<Long> orderIds = errandOrderList.stream().map(ErrandOrder::getOrderId).collect(Collectors.toList());

        // 如果没有符合条件的订单，则返回空
        if (orderIds.isEmpty()) {
            return ResultUtils.success(Collections.emptyList());
        }

        // 查询 orders 表，获取相关订单信息
        LambdaQueryWrapper<Orders> ordersQueryWrapper = new LambdaQueryWrapper<>();
        ordersQueryWrapper.in(Orders::getId, orderIds);
        ordersQueryWrapper.eq(Orders::getLocation, distributionScope);

        // 获取订单数据
        List<Orders> ordersList = ordersService.list(ordersQueryWrapper);
        List<OrdersVO> ordersVO = ordersService.getOrdersVO(ordersList);

        // 如果转换后的订单VO为空，也返回空响应
        if (CollectionUtils.isEmpty(ordersVO)) {
            return ResultUtils.success(Collections.emptyList());
        }

        return ResultUtils.success(ordersVO);
    }

    /**
     * 根据Id获取跑腿信息
     * @return 跑腿信息
     */
    @PostMapping("/get/id")
    @AuthCheck(mustRole = ADMIN_ROLE)
    public BaseResponse<ErrandVO> getErrandById (String errandId1) {
        Long errandId = Long.valueOf(errandId1);
        Errand errand = errandService.getOne(Wrappers.<Errand>lambdaQuery().eq(Errand::getId, errandId));
        ThrowUtils.throwIf(errand == null, ErrorCode.PARAMS_ERROR, "当前跑腿id不正确");
        ErrandVO errandVO = errandService.getErrandVO(errand);
        ErrandAuth errandAuth = errandAuthService.getErrandAuth(errandId);
        errandVO.setBackIdCard(errandAuth.getBackIdCard());
        errandVO.setBankCard(errandAuth.getBankCard());
        errandVO.setFrontIdCard(errandAuth.getFrontIdCard());
        return ResultUtils.success(errandVO);
    }

}
