package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.OrderRefunds;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.refund.RefundAddRequest;
import com.bsz.school_send_back_end.model.dto.refund.RefundUpdateRequest;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.OrderRefundsService;
import com.bsz.school_send_back_end.service.OrdersService;
import com.bsz.school_send_back_end.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Tag(name = "退款接口")
@RestController
@RequestMapping("/refund")
public class RefundController {
    @Resource
    private UserService userService;

    @Resource
    private OrderRefundsService refundsService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private BusinessService businessService;

    /**
     * 用户申请退款
     * @param refundAddRequest 退款请求
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addRefund(@RequestBody RefundAddRequest refundAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long orderId = refundAddRequest.getOrderId();
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Orders::getId, orderId);
        Orders orders = ordersService.getOne(queryWrapper);
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR, "未找到退款申请");
        ThrowUtils.throwIf(orders.getState() != 1, ErrorCode.SYSTEM_ERROR, "该订单未支付");

        OrderRefunds orderRefunds = new OrderRefunds();
        BeanUtils.copyProperties(refundAddRequest, orderRefunds);
        orderRefunds.setUserId(loginUser.getId());

        boolean save = refundsService.save(orderRefunds);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "创建退款单失败");
        return ResultUtils.success(true, "申请退款成功,等待商家确认");
    }

    /**
     * 商家查询退款单
     * @param status 退款单的状态
     * @param request 网络请求
     * @return 退款列表
     */
    @GetMapping("/list/business")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<List<OrderRefunds>> listRefundByBusiness(@RequestParam Integer status, HttpServletRequest request) {
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long id = loginBusiness.getId();
        LambdaQueryWrapper<OrderRefunds> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderRefunds::getStatus, status)
                .inSql(OrderRefunds::getOrderId, "SELECT id FROM orders WHERE businessId = " + id);  // 根据商家ID过滤订单
        List<OrderRefunds> list = refundsService.list(queryWrapper);
        return ResultUtils.success(list);
    }

    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<Boolean> updateRefundByBusiness (@RequestBody RefundUpdateRequest updateRequest, HttpServletRequest request) {
        if (updateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long orderId = updateRequest.getOrderId();
        Integer status = updateRequest.getStatus();
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        Orders orders = ordersService.getOne(Wrappers.<Orders>lambdaQuery()
                .eq(Orders::getId, orderId));
        ThrowUtils.throwIf(orders == null, ErrorCode.PARAMS_ERROR, "传入的订单id不正确");
        ThrowUtils.throwIf(businessId.equals(orders.getBusinessId()), ErrorCode.PARAMS_ERROR, "当前订单不属于该商家");

        LambdaQueryWrapper<OrderRefunds> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderRefunds::getOrderId, orderId);
        OrderRefunds refunds = refundsService.getOne(queryWrapper);
        refunds.setStatus(status); // 1-已同意 2 - 已拒绝
        ThrowUtils.throwIf(status == 2 && updateRequest.getRefuseReason() == null, ErrorCode.PARAMS_ERROR, "请填写退款原因");
        boolean updateById = refundsService.updateById(refunds);
        ThrowUtils.throwIf(!updateById, ErrorCode.OPERATION_ERROR, "修改退款单状态失败");
        return ResultUtils.success(true);
    }

    @PostMapping("/list/reson")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<String>> getReasonByAdmin() {
        // 获取今天的开始和结束时间
        LocalDate today = LocalDate.now();
        LocalDateTime startOfDay = today.atStartOfDay(); // 今天的00:00:00
        LocalDateTime endOfDay = today.atTime(LocalTime.MAX); // 今天的23:59:59.999999999

        // 将 LocalDateTime 转换为 Date
        Date startOfDayDate = Date.from(startOfDay.atZone(ZoneId.systemDefault()).toInstant());
        Date endOfDayDate = Date.from(endOfDay.atZone(ZoneId.systemDefault()).toInstant());
        LambdaQueryWrapper<OrderRefunds> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.ge(OrderRefunds::getCreateTime, startOfDayDate);
        queryWrapper.le(OrderRefunds::getCreateTime, endOfDayDate);
        List<OrderRefunds> refunds = refundsService.list(queryWrapper);
        List<String> collect = refunds.stream()
                .map(OrderRefunds::getReason)
                .collect(Collectors.toList());
        return ResultUtils.success(collect);
    }


}
