package com.bsz.school_send_back_end.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Specifications;
import com.bsz.school_send_back_end.model.domain.SpecificationsDishes;
import com.bsz.school_send_back_end.model.dto.attribute.AttributeAddRequest;
import com.bsz.school_send_back_end.model.dto.attribute.AttributeUpdateRequest;
import com.bsz.school_send_back_end.model.dto.specifications.SpecificationsAddRequest;
import com.bsz.school_send_back_end.model.dto.specifications.SpecificationsMyQueryRequest;
import com.bsz.school_send_back_end.model.dto.specifications.SpecificationsQueryRequest;
import com.bsz.school_send_back_end.model.dto.specifications.SpecificationsUpdateRequest;
import com.bsz.school_send_back_end.model.vo.SpecificationsVO;
import com.bsz.school_send_back_end.service.AttributeService;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.SpecificationsDishesService;
import com.bsz.school_send_back_end.service.SpecificationsService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "规格接口")
@RequestMapping("/specifications")
public class SpecificationsController {

    @Resource
    private SpecificationsService specificationsService;

    @Resource
    private SpecificationsDishesService specificationsDishesService;

    @Resource
    private BusinessService businessService;

    @Resource
    private AttributeService attributeService;

    /**
     * 添加规格
     * @param specificationsAddRequest 添加请求体
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> addSpecifications(@RequestBody SpecificationsAddRequest specificationsAddRequest, HttpServletRequest request){
        if (specificationsAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Specifications specifications = new Specifications();
        BeanUtils.copyProperties(specificationsAddRequest,specifications);
        //校验请求体
        specificationsService.validSpecifications(specifications, false);
        //添加规格和属性
        List<AttributeAddRequest> attributeAddRequests = specificationsAddRequest.getAttributeAddRequests();
        Boolean aBoolean = specificationsService.addSpecifications(specifications, attributeAddRequests, request);
        ThrowUtils.throwIf(!aBoolean, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    /**·
     * 更新规格
     * @param specificationsUpdateRequest 更新请求体
     * @param request 网络请求
     * @return 是否成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> updateSpecifications(@RequestBody SpecificationsUpdateRequest specificationsUpdateRequest, HttpServletRequest request) {
        if (specificationsUpdateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Specifications specifications = new Specifications();
        BeanUtils.copyProperties(specificationsUpdateRequest,specifications);
        //校验
        specificationsService.validSpecifications(specifications,true);

        //更新规格和属性
        List<AttributeUpdateRequest> attributeUpdateRequests = specificationsUpdateRequest.getAttributeUpdateRequests();
        Boolean aBoolean = specificationsService.updateSpecifications(specifications, attributeUpdateRequests, request);
        ThrowUtils.throwIf(!aBoolean, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 根据菜品id获取规格脱敏信息
     * @param specificationsMyQueryRequest 请求体
     * @return 脱敏列表
     */
    @PostMapping("/list/specificationsVOByDishesId")
    public BaseResponse<List<SpecificationsVO>> specificationsVOByDishesId(@RequestBody SpecificationsMyQueryRequest specificationsMyQueryRequest) {
        if (specificationsMyQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long dishesId = specificationsMyQueryRequest.getId();
        //根据id获取中间表数据
        QueryWrapper<SpecificationsDishes> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("dishesId",dishesId);
        List<SpecificationsDishes> list = specificationsDishesService.list(queryWrapper);
        //根据中间表数据中的规格获取规格信息和属性
        return ResultUtils.success(specificationsDishesService.getSpecificationsAddAttribute(list));
    }

    /**
     * 获取脱敏列表
     * @param specificationsQueryRequest 前端请求
     * @param request 网络请求
     * @return 脱敏列表
     */
    @PostMapping("/list/specificationsVO")
    public BaseResponse<List<SpecificationsVO>> specificationsVOList(@RequestBody SpecificationsQueryRequest specificationsQueryRequest, HttpServletRequest request) {
        //获取商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        //获取商家的规格表
        String specificationsName = specificationsQueryRequest.getSpecificationsName();
        QueryWrapper<Specifications> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(specificationsName), "specificationsName", specificationsName);
        queryWrapper.eq("businessId",businessId);
        List<Specifications> list = specificationsService.list(queryWrapper);
        //对规格信息脱敏
        List<SpecificationsVO> specificationsVO = specificationsService.getSpecificationsVO(list);
        //获取各规格的属性脱敏信息
        return ResultUtils.success(attributeService.getBySpecificationsVoId(specificationsVO));
    }
}
