package com.bsz.school_send_back_end.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.domain.UserRating;
import com.bsz.school_send_back_end.model.dto.CommonRequest;
import com.bsz.school_send_back_end.model.dto.userRating.UserRatingAddRequest;
import com.bsz.school_send_back_end.model.dto.userRating.UserRatingReviewRequest;
import com.bsz.school_send_back_end.service.UserRatingService;
import com.bsz.school_send_back_end.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;

@Tag(name = "用户评分接口")
@RestController
@Slf4j
@RequestMapping("/level")
public class UserRatingController {

    @Resource
    private UserRatingService userRatingService;

    @Resource
    private UserService userService;

    /**
     * 添加用户评分
     * @param userRatingAddRequest 用户评分请求
     * @param request 当期那登录用户
     * @return 是否添加成功
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addRating (@RequestBody UserRatingAddRequest userRatingAddRequest, HttpServletRequest request) {
        if (userRatingAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        UserRating userRating = new UserRating();
        BeanUtils.copyProperties(userRatingAddRequest, userRating);
        userRatingService.validUserRating(userRating, request);
        boolean save = userRatingService.save(userRating);
        ThrowUtils.throwIf(!save, ErrorCode.SYSTEM_ERROR, "添加失败");
        return ResultUtils.success(true);
    }

    /**
     * 删除用户评分
     * @param commonRequest 用户评分id
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    public BaseResponse<Boolean> deleteUserRating (@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        boolean remove = userRatingService.removeById(commonRequest.getId());
        ThrowUtils.throwIf(!remove, ErrorCode.SYSTEM_ERROR, "删除失败");

        return ResultUtils.success(true);
    }

    /**
     * 展示当前用户评分
     * @param request 当前用户
     * @return 评分列表
     */
    @PostMapping("/list")
    public BaseResponse<List<UserRating>> listUserRating (@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long ratedEntityType = commonRequest.getId();
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        List<UserRating> list = userRatingService.list(Wrappers.<UserRating>lambdaQuery()
                .eq(UserRating::getUserId, userId).eq(UserRating::getRatedEntityType, ratedEntityType) );
        return ResultUtils.success(list);
    }

    /**
     * 展示当前用户评分
     * @param request 当前用户
     * @return 评分列表
     */
    @PostMapping("/list/user")
    public BaseResponse<List<UserRating>> listUserRatingByUser (HttpServletRequest request) {
        List<UserRating> userRatings = userRatingService.listUserRatingByUser(request);
        return ResultUtils.success(userRatings);
    }

    @PostMapping("/business/reply")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<Boolean> businessReply (@RequestBody UserRatingReviewRequest userRatingReviewRequest, HttpServletRequest request) {
        if (userRatingReviewRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = userRatingReviewRequest.getId();
        String businessReview = userRatingReviewRequest.getBusinessReview();
        userRatingService.validUserRatingByBusiness(id, request);
        LambdaQueryWrapper<UserRating> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(UserRating::getId, id);
        UserRating userRating = userRatingService.getOne(wrapper);
        userRating.setBusinessReview(businessReview);
        boolean update = userRatingService.updateById(userRating);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    @PostMapping("/list/business")
    public BaseResponse<List<UserRating>> listUserRatingByBusiness (@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long businessId = commonRequest.getId();

        List<UserRating> list = userRatingService.list(Wrappers.<UserRating>lambdaQuery()
                .eq(UserRating::getRatedEntityId, businessId));
        return ResultUtils.success(list);
    }
}
