package com.bsz.school_send_back_end.listener;



import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.RabbitMQConstant;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.domain.PrivateMessage;
import com.bsz.school_send_back_end.service.OrdersService;
import com.bsz.school_send_back_end.utils.MultiDelayMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;

import jakarta.annotation.Resource;


@Component
@Slf4j
public class RabbitMQReceiver {
    @Resource
    private OrdersService ordersService;

    @Resource
    private RabbitTemplate rabbitTemplate;

    @RabbitListener(queues = RabbitMQConstant.TOPIC_QUEUE2)
    public Orders receiveOrder (Orders message) {
        log.info("接收数据：" + message);
        return message;
    }



    @RabbitListener(queues = RabbitMQConstant.TOPIC_QUEUE1)
    public PrivateMessage receive (PrivateMessage message) {
        log.info("接收数据：" + message);
        return message;

    }

    /**
     * 过三分钟未支付自动取消
     * @param msg
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(RabbitMQConstant.CANCEL_ORDER_QUEUE),
            exchange = @Exchange(name = RabbitMQConstant.CANCEL_ORDER_EXCHANGE, delayed = "true"),
            key = RabbitMQConstant.CANCEL_ORDER_ROUTING_KEY
    ))
    public void handleExpiredOrder(MultiDelayMessage<Long> msg) {
        Long orderId = msg.getData();
        log.info("订单超时检查，订单ID：{}", orderId);

        // 查询订单
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Orders::getId, orderId);
        Orders orders = ordersService.getOne(queryWrapper);

        if (orders == null) {
            log.warn("订单不存在，订单ID：{}", orderId);
            return;
        }

        if (orders.getState() == 0) { // 订单仍未支付
            if (msg.hasNextDelay()) {
                // 还有下一个检查点，继续发送延迟消息
                int nextDelay = msg.removeNextDelay().intValue();
                rabbitTemplate.convertAndSend(RabbitMQConstant.CANCEL_ORDER_EXCHANGE,
                        RabbitMQConstant.CANCEL_ORDER_ROUTING_KEY, msg, message -> {
                            message.getMessageProperties().setDelay(nextDelay);
                            return message;
                        });
                log.info("订单未支付，下一次检查延迟 {} 毫秒，订单ID：{}", nextDelay, orderId);
            } else {
                // **所有检查点都执行完，才执行订单取消**
                log.info("订单超时未支付，自动取消，订单ID：{}", orderId);
                orders.setState(3); // 订单状态修改为已取消（假设 3 代表“已取消”）
                boolean update = ordersService.updateById(orders);
                ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单取消失败");
                log.info("订单超时，已取消，订单ID：{}", orderId);
            }
        } else {
            log.info("订单已支付，无需取消，订单ID：{}", orderId);
        }
    }

    /**
     * 过三十分钟无人接单自动退款
     * @param msg
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(RabbitMQConstant.DELIVERY_ORDER_QUEUE),
            exchange = @Exchange(name = RabbitMQConstant.DELIVERY_DELAY_EXCHANGE, delayed = "true"),
            key = RabbitMQConstant.DELIVERY_ORDER_ROUTING_KEY
    ))
    public void handleExpiredDeliveryOrder(MultiDelayMessage<Long> msg) {
        Long orderId = msg.getData();
        log.info("外卖单超时检查，订单ID：{}", orderId);

        // 查询订单
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Orders::getId, orderId);
        Orders orders = ordersService.getOne(queryWrapper);

        if (orders == null) {
            log.warn("外卖订单不存在，订单ID：{}", orderId);
            return;
        }

        if (orders.getState() == 1) { // 外卖单仍未支付
            // 外卖单超时退款逻辑
            if (msg.hasNextDelay()) {
                int nextDelay = msg.removeNextDelay().intValue();
                rabbitTemplate.convertAndSend(RabbitMQConstant.DELIVERY_DELAY_EXCHANGE,
                        RabbitMQConstant.DELIVERY_ORDER_ROUTING_KEY, msg, message -> {
                            message.getMessageProperties().setDelay(nextDelay);
                            return message;
                        });
                log.info("外卖单未接单，下一次检查延迟 {} 毫秒，订单ID：{}", nextDelay, orderId);
            } else {
                // 超过30分钟，未接单，自动退款
                log.info("外卖单超时未接单，自动退款，订单ID：{}", orderId);
                orders.setState(2); // 订单状态修改为已退款
                boolean update = ordersService.updateById(orders);
                ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "外卖单退款修改失败");
                log.info("外卖单超时，退款成功，订单ID：{}", orderId);
            }
        } else {
            log.info("外卖单已支付，无需退款，订单ID：{}", orderId);
        }
    }



}