package com.bsz.school_send_back_end.manager;

import cn.hutool.core.util.StrUtil;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.domain.UserLoginRedisInfo;
import com.bsz.school_send_back_end.service.UserService;
import com.bsz.school_send_back_end.utils.NetUtils;
import com.bsz.school_send_back_end.utils.RedisKeyUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.session.data.redis.RedisSessionRepository;
import org.springframework.stereotype.Component;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;

import java.util.concurrent.TimeUnit;

import static com.bsz.school_send_back_end.contant.RedisKeyConstant.*;

@Slf4j
@Component
public class SessionManager {

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Resource
    private RedisSessionRepository sessionRepository; // 正确，类型匹配

    @Value("${spring.session.timeout}")
    private long sessionTimeout;

    @Lazy
    @Resource
    private UserService userService;

    /**
     * 登录
     */
    public String login (User user, String appName, HttpServletRequest request) {
        String message = "登录成功";
        String ipAddress = NetUtils.getIpAddress(request);
        String oldSessionId = this.checkOtherLogin(user.getId(), ipAddress, request);
        //不为空，表示在其他端以登录
        if (StrUtil.isNotBlank(oldSessionId)) {
            //删除其他端的登录态
            this.removeOtherSessionAttribute(oldSessionId, user.getId());
            message += ",已移除其他设备的登录";
        }

        UserLoginRedisInfo build = UserLoginRedisInfo.builder()
                .user(user)
                .ip(ipAddress)
                .appName(appName).
                build();
        this.setLoginAttribute(request, USER_LOGIN_STATE, build);
        return message;
    }

    /**
     *  退出登录
     */
    public void logout (HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        removeAttribute(request, USER_LOGIN_STATE);
        stringRedisTemplate.delete(RedisKeyUtils.getUserExtraInfoKey(loginUser.getId()));
    }
    /**
     * 检查是否在其他端登录
     */
    public String checkOtherLogin(Long userId, String currentIp, HttpServletRequest request) {
        //校验sessionId
        Object oldSessionIdObj = stringRedisTemplate.opsForHash().get(RedisKeyUtils.getUserExtraInfoKey(userId), SESSION_ID);
        String oldSessionId = null;
        if (oldSessionIdObj != null) {
            oldSessionId = (String) oldSessionIdObj;
        }
        //校验ip
        Object oldIpObj = stringRedisTemplate.opsForHash().get(RedisKeyUtils.getUserExtraInfoKey(userId), IP);
        String oldIP = null;
        if (oldIpObj != null) {
            oldIP = (String) oldIpObj;
        }

        // 判断sessionId如果为空或相等，返回null；不等，判断ip
        // 如果ip为空或相等，返回null，不等，返回oldSessionId
        if (StrUtil.isBlank(oldSessionId) || oldSessionId.equals(request.getSession().getId())) {
            return null;
        }
        else {
            if (StrUtil.isBlank(oldIP) || oldIP.equals(currentIp)) {
                return null;
            }
        }

        return oldSessionId;
    }

    /**
     * 删除其他设备登录态
     */
    public void removeOtherSessionAttribute (String sessionId, Long userId) {
        String sessionKey = RedisKeyUtils.getSessionKey(sessionId);
        String sessionAttrKey = RedisKeyUtils.getSessionAttrKey(USER_LOGIN_STATE);
        //删除用户的额外信息
        Boolean userExtraDelete = stringRedisTemplate.delete(RedisKeyUtils.getUserExtraInfoKey(userId));
        Long delete = sessionRepository.getSessionRedisOperations().opsForHash().delete(sessionKey, sessionAttrKey);
        log.info("oldSessionId: {}，user extra info delete result: {}，user login state delete result: {}",
                sessionId, userExtraDelete, delete);
    }

    /**
     * 设置登录属性
     */
    public void setLoginAttribute(HttpServletRequest request, String loginKey, UserLoginRedisInfo userLoginRedisInfo) {
        setAttribute(request, loginKey, userLoginRedisInfo, true);
    }

    /**
     * 设置属性
     */

    public void setAttribute(HttpServletRequest request, String key, Object value, boolean login) {
        HttpSession session = request.getSession();
        if (login) {
            UserLoginRedisInfo userLoginRedisInfo = (UserLoginRedisInfo) value;
            User user = userLoginRedisInfo.getUser();
            // 存储登录状态
            session.setAttribute(key, user);
            //存储sessionId和ip信息
            String sessionId = session.getId();
            String userExtraInfoKey = RedisKeyUtils.getUserExtraInfoKey(user.getId());
            stringRedisTemplate.opsForHash().put(userExtraInfoKey, SESSION_ID, sessionId);
            stringRedisTemplate.opsForHash().put(userExtraInfoKey, IP, userLoginRedisInfo.getIp());
            stringRedisTemplate.opsForHash().put(userExtraInfoKey, APPNAME,userLoginRedisInfo.getAppName());
            stringRedisTemplate.expire(userExtraInfoKey, sessionTimeout, TimeUnit.SECONDS);
        }
        else {
            session.setAttribute(key, value);
        }
    }

    /**
     * 删除属性
     */
    public void removeAttribute(HttpServletRequest request, String key) {
        HttpSession session = request.getSession();
        session.removeAttribute(key);
    }

}
