package com.bsz.school_send_back_end.service;

import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.domain.PrivateMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import jakarta.annotation.Resource;

@Service
@Slf4j
public class WebsocketService {

    @Resource
    private SimpMessagingTemplate template;

    @Resource
    private PrivateMessageService privateMessageService;

    @Resource
    private BusinessService businessService;

    private static final String ERRAND_TOPIC_PREFIX = "/topic/ServerToClient.errand.";
    /**
     * 简单点对点聊天
     */
    public void sendChatMessage(PrivateMessage message) {
        if (message == null || message.getTo_userAccount() == null || message.getMessage() == null) {
            log.error("发送消息失败，消息内容不完整: {}", message);
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "消息内容不完整");
        }

        log.info("正在发送消息: {}", message);
        message.setMessage(message.getFrom_userAccount() + "发送：" + message.getMessage());

        // 推送消息
        String destination = "/topic/ServerToClient.private." + message.getTo_userAccount();
        template.convertAndSend(destination, message);

        // 存储消息到数据库
        boolean save = privateMessageService.save(message);
        if (!save) {
            log.error("消息存储失败: {}", message);
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "消息存储失败");
        }
        log.info("消息发送成功并存储: {}", message);
    }

    /**
     * 给商家传送订单信息
     */
    public void sendOrderMessage(Orders orders) {
        if (orders == null || orders.getBusinessId() == null) {
            log.error("订单信息不完整，无法推送: {}", orders);
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "订单信息不完整");
        }

        log.info("正在推送订单信息: {}", orders);
        Long businessId = orders.getBusinessId();
        Business business = businessService.getById(businessId);

        if (business == null || business.getUserId() == null) {
            log.error("商家信息未找到，商家ID: {}", businessId);
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "商家信息未找到");
        }

        Long userId = business.getUserId();
        String destination = "/topic/ServerToClient.order." + userId;
        template.convertAndSend(destination, orders);

        log.info("订单信息推送成功，目标用户ID: {}", userId);
    }

    /**
     * 商家将订单发送给跑腿
     */
    @Async
    public void sendOrderToErrand(Orders orders) {
        if (orders == null || orders.getLocation() == null || orders.getLocation().trim().isEmpty()) {
            log.error("订单信息或跑腿位置不完整，无法推送: {}", orders);
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "订单信息或位置信息不完整");
        }

        String location = orders.getLocation();

        // 校验位置是否有效
        if (!UserConstant.VALID_LOCATIONS.contains(location)) {
            log.error("非法位置，无法推送订单: 位置 = {}", location);
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "位置不合法");
        }

        log.info("正在向跑腿推送订单信息, 订单ID: {}, 位置: {}", orders.getId(), location);
        String destination = ERRAND_TOPIC_PREFIX + location;
        template.convertAndSend(destination, orders);

        log.info("订单推送成功，位置: {}", location);
    }

}
