package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.BusinessAuth;
import com.bsz.school_send_back_end.model.domain.BusinessLevel;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.business.BusinessQueryRequest;
import com.bsz.school_send_back_end.model.vo.BusinessVO;
import com.bsz.school_send_back_end.service.BusinessAuthService;
import com.bsz.school_send_back_end.service.BusinessLevelService;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.mapper.BusinessMapper;
import com.bsz.school_send_back_end.service.UserService;
import com.bsz.school_send_back_end.utils.RegexUtils;
import com.bsz.school_send_back_end.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* @author xy
*/
@Service
public class BusinessServiceImpl extends ServiceImpl<BusinessMapper, Business>
    implements BusinessService{

    @Resource
    private UserService userService;


    @Resource
    private BusinessAuthService businessAuthService;

    @Resource
    private BusinessLevelService businessLevelService;

    @Resource
    private BusinessMapper businessMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized void addBusiness(User user, Business business, BusinessAuth businessAuth) {
        boolean save = userService.save(user);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "创建商家失败");

        //商家绑定user
        business.setUserId(user.getId());
        boolean save1 = this.save(business);
        ThrowUtils.throwIf(!save1, ErrorCode.OPERATION_ERROR, "绑定id失败");
        //添加认证信息
        businessAuth.setBusinessId(business.getId());
        boolean save2 = businessAuthService.save(businessAuth);
        ThrowUtils.throwIf(!save2, ErrorCode.OPERATION_ERROR, "商家认证信息失败");

        businessLevelService.updateBusinessLevel(business.getId());

    }

    @Override
    public void validBusiness(Business business, boolean add) {
        String businessName = business.getBusinessName();
        String businessAvatar = business.getBusinessAvatar();
        String businessPhone = business.getBusinessPhone();
        String address = business.getAddress();
        String startBusiness = business.getStartBusiness();
        String endBusiness = business.getEndBusiness();

        //只在添加时严格校验 保证商家信息是全的
        if (add) {
            if (StringUtils.isAnyBlank(businessName, businessAvatar, businessPhone, address, startBusiness, endBusiness)) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "商家信息不全");
            }
        }

        // 有参数可以校验参数 保证商家名字唯一
        if (StringUtils.isNotBlank(businessName)) {
            QueryWrapper<Business> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("businessName", businessName);
            long count = this.count(queryWrapper);
            ThrowUtils.throwIf(count > 0, ErrorCode.OPERATION_ERROR, "门店名称重复");
        }

        //校验手机号格式
        if (StringUtils.isNotBlank(businessPhone)) {
            boolean phoneInvalid = RegexUtils.isPhoneInvalid(businessPhone);
            ThrowUtils.throwIf(phoneInvalid, ErrorCode.PARAMS_ERROR, "手机号格式错误");

            QueryWrapper<Business> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("businessPhone", businessPhone);
            long count = this.count(queryWrapper);
            ThrowUtils.throwIf(count > 0 , ErrorCode.OPERATION_ERROR, "手机号重复");
        }
    }

    @Override
    public void validBusinessAuth(BusinessAuth businessAuth, boolean add) {
        //只在添加时严格校验 保证商家认证信息是全的
        if (add) {
            String shopkeeper = businessAuth.getShopkeeper();
            String license = businessAuth.getLicense();
            String frontIdCard = businessAuth.getFrontIdCard();
            String backIdCard = businessAuth.getBackIdCard();
            String bankCard = businessAuth.getBankCard();
            if (StringUtils.isAnyBlank(shopkeeper, license, frontIdCard, bankCard, backIdCard)) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "商家认证信息不全");
            }
        }
    }

    @Override
    public QueryWrapper<Business> getQueryWrapper(BusinessQueryRequest businessQueryRequest) {

        if (businessQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }

        Long id = businessQueryRequest.getId();
        Long userId = businessQueryRequest.getUserId();
        Long categoryId = businessQueryRequest.getCategoryId();
        Integer state = businessQueryRequest.getState();
        Integer storeStatus = businessQueryRequest.getStoreStatus();
        String businessName = businessQueryRequest.getBusinessName();
        String address = businessQueryRequest.getAddress();
        String businessProfile = businessQueryRequest.getBusinessProfile();
        String sortField = businessQueryRequest.getSortField();
        String sortOrder = businessQueryRequest.getSortOrder();
        QueryWrapper<Business> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(categoryId), "categoryId", categoryId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(state), "state", state);
        queryWrapper.eq(ObjectUtils.isNotEmpty(storeStatus), "storeStatus", storeStatus);
        queryWrapper.like(StringUtils.isNotBlank(businessName), "businessName", businessName);
        queryWrapper.like(StringUtils.isNotBlank(address), "address", address);
        queryWrapper.like(StringUtils.isNotBlank(businessProfile), "businessProfile", businessProfile);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }

    @Override
    public BusinessVO getBusinessVO(Business business) {
            BusinessVO businessVO = new BusinessVO();
        if (business == null) {
            return null;
        }
        BeanUtils.copyProperties(business,businessVO);
        BusinessLevel businessLevel = businessLevelService.getOne(Wrappers.<BusinessLevel>lambdaQuery()
                .eq(BusinessLevel::getBusinessId, business.getId()));
        if (businessLevel == null) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR,"当前商家没有等级");
        }
        Integer level = businessLevel.getLevel();
        businessVO.setLevel(level);
        return businessVO;
    }

    @Override
    public List<BusinessVO> getBusinessVO(List<Business> businessList) {
        if (CollectionUtils.isEmpty(businessList)) {
            return new ArrayList<>();
        }
        return businessList.stream().map(business -> {
            BusinessVO businessVO = new BusinessVO();
            BeanUtils.copyProperties(business, businessVO);
            BusinessLevel businessLevel = businessLevelService.getOne(Wrappers.<BusinessLevel>lambdaQuery()
                    .eq(BusinessLevel::getBusinessId, business.getId()));
            businessVO.setLevel(businessLevel.getLevel());
            return businessVO;
        }).collect(Collectors.toList());
    }

    @Override
    public Business getLoginBusiness(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        QueryWrapper<Business> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId",loginUser.getId());
        Business business = this.baseMapper.selectOne(queryWrapper);
        ThrowUtils.throwIf(business == null,ErrorCode.NOT_FOUND_ERROR,"当前用户没有关联商家");
        return business;
    }

    @Override
    public List<BusinessVO> findAllBusiness() {
        List<BusinessLevel> businessLevels = businessLevelService.findAllOrdered();
        // 获取商家 ID 列表
        List<Long> businessIds = businessLevels.stream()
                .map(BusinessLevel::getBusinessId)
                .distinct()
                .collect(Collectors.toList());
        List<Business> businesses = businessMapper.selectBatchIds(businessIds);

        //将商家等级信息映射到商家实体
        Map<Long, Integer> levelMap = businessLevels.stream()
                .collect(Collectors.toMap(BusinessLevel::getBusinessId, BusinessLevel::getLevel));
        // Step 5: 构建 BusinessVO 并将等级添加到商家数据中
        List<BusinessVO> businessVOs = businesses.stream()
                .map(business -> {
                    BusinessVO vo = new BusinessVO();
                    BeanUtils.copyProperties(business, vo);  // 自动复制 Business 到 BusinessVO
                    vo.setLevel(levelMap.getOrDefault(business.getId(), 1)); // 设置等级
                    return vo;
                })
                // Step 6: 按等级排序
                .sorted(Comparator.comparingLong(BusinessVO::getLevel).reversed()) // 按等级降序排序
                .collect(Collectors.toList());

        return businessVOs;
    }


}