package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.Cart;
import com.bsz.school_send_back_end.model.domain.Dishes;
import com.bsz.school_send_back_end.model.domain.User;
import com.bsz.school_send_back_end.model.dto.cart.CartAddRequest;
import com.bsz.school_send_back_end.model.dto.cart.CartUpdateRequest;
import com.bsz.school_send_back_end.model.vo.CartVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.CartService;
import com.bsz.school_send_back_end.mapper.CartMapper;
import com.bsz.school_send_back_end.service.DishesService;
import com.bsz.school_send_back_end.service.UserService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author xy
*/
@Service
public class CartServiceImpl extends ServiceImpl<CartMapper, Cart>
    implements CartService{

    @Resource
    private BusinessService businessService;

    @Resource
    private DishesService dishesService;

    @Resource
    private UserService userService;

    @Override
    public void validCart(CartAddRequest cartAddRequest) {
        if (cartAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Long businessId = cartAddRequest.getBusinessId();
        Long dishesId = cartAddRequest.getDishesId();
        Integer quantity = cartAddRequest.getQuantity();

        if (quantity == null || quantity < 1) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR,"菜品数量错误");
        }

        Business business = businessService.getById(businessId);
        if (business == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"不存在改商家");
        }

        Dishes dishes = dishesService.getById(dishesId);
        if (dishes == null || dishes.getStatus().equals("下架")){
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"不存在改菜品");
        }
    }

    @Override
    public Long addCart(CartAddRequest cartAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Long businessId = cartAddRequest.getBusinessId();
        Long dishesId = cartAddRequest.getDishesId();
        //计算菜品价格
        Dishes dishes = dishesService.getById(dishesId);
        BigDecimal price = new BigDecimal(String.valueOf(dishes.getDishesPrice()));
        //保存购物车菜品
        Cart cart = new Cart();
        cart.setBusinessId(businessId);
        cart.setUserId(userId);
        cart.setPrice(price);
        cart.setDishesId(dishesId);
        boolean save = this.save(cart);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        return cart.getId();
    }

    @Override
    public Boolean updateCart(CartUpdateRequest cartUpdateRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Long dishesId = cartUpdateRequest.getDishesId();
        String selectedOptions = cartUpdateRequest.getSelectedOptions();
        Long id = cartUpdateRequest.getId();
        ThrowUtils.throwIf(id == null,ErrorCode.OPERATION_ERROR,"购物车id不存在");
        //检验该购物车是否为登录用户的
        Cart cartId =this.getById(id);
        Long cartUserId = cartId.getUserId();
        if (userId.equals(cartUserId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"无法操作该购物车");
        }
        Dishes dishes = dishesService.getById(dishesId);
        BigDecimal price = new BigDecimal(String.valueOf(dishes.getDishesPrice()));
        LambdaQueryWrapper<Cart> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Cart::getId, id);
        Cart cart = this.getOne(wrapper);
        Integer quantity = cartUpdateRequest.getQuantity();
        if (cartUpdateRequest.getIsSelectedOptions()) {
            cart.setQuantity(cart.getQuantity() + quantity);
        } else {
            cart.setQuantity(quantity);
        }
        cart.setPrice(price);
        cart.setSubtotal(null);
        cart.setSelectedOptions(selectedOptions);

        return this.updateById(cart);
    }

    @Override
    public Boolean deleteCart(Long cartId, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Cart cart = this.getById(cartId);
        Long cartUserId = cart.getUserId();

        if (!userId.equals(cartUserId)) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "该用户无权删除该购物车");
        }
        boolean remove = this.removeById(cartId);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);

        return true;
    }

    @Override
    public List<CartVO> getCartVOList(List<Cart> cartList) {
        if (CollectionUtils.isEmpty(cartList)) {
            return new ArrayList<>();
        }

        return cartList.stream().map(item ->{
            CartVO cartVO = new CartVO();
            BeanUtils.copyProperties(item, cartVO);
            Business business = businessService.getById(item.getBusinessId());
            Dishes dishes = dishesService.getById(item.getDishesId());
            cartVO.setBusiness(business);
            cartVO.setDishes(dishes);
            return cartVO;
        }).collect(Collectors.toList());
    }
}




