package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.mapper.OrdersMapper;
import com.bsz.school_send_back_end.model.domain.*;
import com.bsz.school_send_back_end.model.dto.order.OrderQueryRequest;
import com.bsz.school_send_back_end.model.dto.order.OrderUpdateRequest;
import com.bsz.school_send_back_end.model.vo.*;
import com.bsz.school_send_back_end.service.*;
import com.bsz.school_send_back_end.utils.RegexUtils;
import com.bsz.school_send_back_end.utils.SqlUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.bsz.school_send_back_end.contant.UserConstant.DELIVERY_FEE_MAP;

/**
 * @author xy
 */
@Service
@Slf4j
public class OrdersServiceImpl extends ServiceImpl<OrdersMapper, Orders>
        implements OrdersService {

    @Resource
    private BusinessService businessService;

    @Resource
    private OrderDetailsService orderDetailsService;

    @Resource
    private UserService userService;

    @Resource
    private DishesService dishesService;

    @Resource
    private ErrandOrderService errandOrderService;

    @Resource
    private OrderImageService orderImageService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized long addOrder(Orders orders, List<OrderDetails> orderDetailsList) {
        boolean saveOrder = this.save(orders);
        if (!saveOrder) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "创建订单失败，数据库错误");
        }
        // 创建订单菜品
        for (OrderDetails orderDetails : orderDetailsList) {
            // 绑定订单和订单详情
            orderDetails.setOrderId(orders.getId());
            boolean saveDetail = orderDetailsService.save(orderDetails);
            if (!saveDetail) {
                log.error("创建订单菜品失败，订单id：" + orders.getId());
                throw new BusinessException(ErrorCode.OPERATION_ERROR, "订单菜品创建失败");
            }
        }
        return orders.getId();
    }

    @Override
    public void validOrder(Orders orders) {
        String userName = orders.getUserName();
        String phone = orders.getPhone();
        Long userId = orders.getUserId();
        Long businessId = orders.getBusinessId();
        BigDecimal totalPrice = orders.getTotalPrice();
        Integer pickupMethod = orders.getPickupMethod();
        Integer payMethod = orders.getPayMethod();
        if (StringUtils.isAnyBlank(userName, phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isNotBlank(phone) && RegexUtils.isPhoneInvalid(phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "手机号格式错误");
        }
        if (ObjectUtils.isEmpty(totalPrice) || totalPrice.floatValue() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.anyNull(userId, businessId, payMethod, pickupMethod)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
    }

    @Override
    public OrdersVO getOrdersVO(Orders orders) {
        if (orders == null) {
            return null;
        }
        OrdersVO ordersVO = new OrdersVO();
        BeanUtils.copyProperties(orders, ordersVO);
        // 关联查询商家信息
        Long businessId = orders.getBusinessId();
        Business business = this.getBusiness(businessId);
        BusinessVO businessVO = businessService.getBusinessVO(business);
        ordersVO.setBusinessVO(businessVO);
        // 关联查询订单详情信息
        Long orderId = orders.getId();
        QueryWrapper<OrderDetails> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", orderId);
        List<OrderDetails> orderDetailsList = orderDetailsService.list(queryWrapper);
        List<OrderDetailsVO> orderDetailsVOList = orderDetailsService.getOrderDetailsVO(orderDetailsList);
        ordersVO.setOrderDetailsVOList(orderDetailsVOList);
        // 关联查询评价状态

        // 设置商品数量
        Integer number = 0;
        for (OrderDetailsVO detailsVO : orderDetailsVOList) {
            number += detailsVO.getQuantity();
        }
        String address = business.getAddress();
        ordersVO.setAddress(address);
        ordersVO.setNumber(number);

        ErrandOrder one = errandOrderService.getOne(Wrappers.<ErrandOrder>lambdaQuery()
                .eq(ErrandOrder::getOrderId, orderId));
        if (one != null) {
            ordersVO.setPickupEndTime(one.getEndTime());
            ordersVO.setPickupStartTime(one.getStartTime());
        }

        OrderImage image = orderImageService.getOne(Wrappers.<OrderImage>lambdaQuery()
                .eq(OrderImage::getOrderId, orderId));
        if (image != null) {
            ordersVO.setImageAddress(image.getImageAddress());
        }
        return ordersVO;
    }

    @Override
    public List<OrdersVO> getOrdersVO(List<Orders> ordersList) {
        if (CollectionUtils.isEmpty(ordersList)) {
            return new ArrayList<>();
        }
//        return ordersList.stream().map(this::getOrdersVO).collect(Collectors.toList());
        return ordersList.stream().map(orders ->{
            OrdersVO ordersVO = this.getOrdersVO(orders);
            Business business = this.getBusiness(orders.getBusinessId());
            ordersVO.setAddress(business.getAddress());
            return ordersVO;
        }).collect(Collectors.toList());
    }

    @Override
    public QueryWrapper<Orders> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        Long id = orderQueryRequest.getId();
        Long userId = orderQueryRequest.getUserId();
        Long businessId = orderQueryRequest.getBusinessId();
        Long errandId = orderQueryRequest.getErrandId();
        String pickupCode = orderQueryRequest.getPickupCode();
        Integer pickupMethod = orderQueryRequest.getPickupMethod();
        Integer state = orderQueryRequest.getState();
        String startTime = orderQueryRequest.getStartTime();
        String endTime = orderQueryRequest.getEndTime();
        String sortField = orderQueryRequest.getSortField();
        String sortOrder = orderQueryRequest.getSortOrder();
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(businessId), "businessId", businessId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(errandId), "errandId", errandId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(pickupMethod), "pickupMethod", pickupMethod);
        queryWrapper.eq(ObjectUtils.isNotEmpty(state), "state", state);
        queryWrapper.ge(StringUtils.isNotBlank(pickupCode), "pickupCode", pickupCode);
        queryWrapper.ge(StringUtils.isNotBlank(startTime), "createTime", startTime);
        queryWrapper.le(StringUtils.isNotBlank(endTime), "createTime", endTime);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }

    @Override
    @Transactional
    public Orders getOrderWithPessimisticLock(Long ordersId) {
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Orders::getId, ordersId);

        // 使用悲观锁，查询并锁定订单
        Orders orders = this.getBaseMapper().selectOne(queryWrapper.last("FOR UPDATE"));

        return orders;
    }

    @Override
    public Boolean OrderToUpdate(OrderUpdateRequest orderUpdateRequest, HttpServletRequest request) {
        if (orderUpdateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Orders orders = this.getById(orderUpdateRequest.getOrderId());
        if (orders == null) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        }
        Integer state = orders.getState();
        if (state != 0) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "当前订单已支付");
        }

        User loginUser = userService.getLoginUser(request);
        ThrowUtils.throwIf(!loginUser.getId().equals(orders.getUserId()), ErrorCode.OPERATION_ERROR, "当前订单不属于该用户");

        ErrandOrder errandOrder = errandOrderService.getOne(Wrappers.<ErrandOrder>lambdaQuery()
                .eq(ErrandOrder::getOrderId, orders.getId()));
        Integer pickupMethod = orderUpdateRequest.getPickupMethod();
        LocalDateTime  pickupStartTime = orderUpdateRequest.getPickupStartTime();
        LocalDateTime  pickupEndTime = orderUpdateRequest.getPickupEndTime();
        String location = orderUpdateRequest.getLocation();
        String notes = orderUpdateRequest.getNotes();
        BigDecimal totalPrice = orders.getTotalPrice();
        Integer currentPickupMethod = orders.getPickupMethod();

        if (pickupMethod == 0) {
            // 转为自取，需要减去相应的费用
            if (currentPickupMethod == 1 || currentPickupMethod == 2) {
                BigDecimal newTotalPrice = calculateTotalPrice(orders.getId(), totalPrice, false);  // 减去打包费
                orders.setTotalPrice(newTotalPrice);
                if (currentPickupMethod == 2) {
                    BigDecimal total = calculateTotal(orders.getId(), newTotalPrice, location, false); //减去配送费
                    orders.setTotalPrice(total);
                }
            }
        } else if (pickupMethod == 1) {
            // 转为外带
            if (currentPickupMethod == 0) {
                // 从自取转为外带，添加打包费
                BigDecimal newTotalPrice = calculateTotalPrice(orders.getId(), totalPrice, true);
                orders.setTotalPrice(newTotalPrice);
            } else if (currentPickupMethod == 2) {
                // 从配送转为外带，减去配送费，保留打包费
                BigDecimal newTotalPrice = calculateTotal(orders.getId(), totalPrice, location, false);
                orders.setTotalPrice(newTotalPrice);
            }
        } else if (pickupMethod == 2) {
            // 转为配送
            if (currentPickupMethod == 0) {
                // 从自取转为配送，添加打包费和配送费
                BigDecimal newTotalPrice = calculateTotalPrice(orders.getId(), totalPrice, true);  // 添加打包费
                BigDecimal total = calculateTotal(orders.getId(), newTotalPrice, location, true);  // 添加配送费
                orders.setTotalPrice(total);
            } else if (currentPickupMethod == 1) {
                // 从外带转为配送，添加配送费，保留打包费
                BigDecimal total = calculateTotal(orders.getId(), totalPrice, location, true);
                orders.setTotalPrice(total);
            }

            // 验证取餐时间和地址
            this.validateDelivery(location, pickupStartTime);

            orders.setLocation(location);  // 设置取餐地点
            long count = errandOrderService.count(Wrappers.<ErrandOrder>lambdaQuery()
                    .eq(ErrandOrder::getOrderId, orders.getId()));
            if (count > 0) {
                errandOrder.setErrandState(1);
                errandOrder.setStartTime(pickupStartTime);
                errandOrder.setEndTime(pickupEndTime);
                boolean update = errandOrderService.updateById(errandOrder);
                ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "更新跑腿订单失败");
            } else {
                ErrandOrder errandOrder1 = new ErrandOrder();
                errandOrder1.setErrandState(1); // 待抢单
                errandOrder1.setOrderId(orders.getId());
                errandOrder1.setStartTime(pickupStartTime);
                errandOrder1.setEndTime(pickupEndTime);
                boolean save = errandOrderService.saveOrUpdate(errandOrder1);
                ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "创建跑腿订单关联失败");
            }
        }

        orders.setPickupMethod(pickupMethod);
        if (pickupMethod == 0 || pickupMethod == 1) {
            orders.setLocation(null);
            if (errandOrder != null) {
                boolean remove = errandOrderService.removeById(errandOrder);
                ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "删除跑腿订单关联失败");
            }
        }

        // 如果备注不为空，则设置备注
        if (StringUtils.isNotBlank(notes)) {
            orders.setNotes(notes);
        }

        // 更新订单
        System.out.println("此时location：" + orders.getLocation());
        ThrowUtils.throwIf(!this.updateById(orders), ErrorCode.OPERATION_ERROR, "修改订单失败");

        return true;
    }


    @Override
    public List<Orders> findExpiredOrders(LocalDateTime deadline) {
        return this.lambdaQuery()
            .in(Orders::getState, 4)
            .lt(Orders::getUpdateTime, deadline) // 更新时间小于deadline（过期）
            .list();
    }

    @Override
    public List<Orders> findCancelOrders(Date halfHourAgo) {
        return this.lambdaQuery()
                .in(Orders::getState, 1)
                .lt(Orders::getUpdateTime, halfHourAgo)
                .eq(Orders::getPickupMethod, 2) // 外卖
                .list();
    }

    @Override
    public OrderErrandVO getOrderErrandVO(Orders orders) {
        if (orders == null) {
            return null;
        }
        Long ordersId = orders.getId();
        OrderErrandVO orderErrandVO = new OrderErrandVO();
        //取餐码
        orderErrandVO.setPickupCode(orders.getPickupCode());
        //图片
        OrderImage image = orderImageService.getOne(Wrappers.<OrderImage>lambdaQuery()
                .eq(OrderImage::getOrderId, ordersId));
        orderErrandVO.setImageAddress(image.getImageAddress());
        //门店名称
        Business business = businessService.getOne(Wrappers.<Business>lambdaQuery()
                .eq(Business::getId, orders.getBusinessId()));
        orderErrandVO.setBusinessName(business.getBusinessName());
        //送达时间
        ErrandOrder errandOrder = errandOrderService.getOne(Wrappers.<ErrandOrder>lambdaQuery()
                .eq(ErrandOrder::getOrderId, ordersId));
        orderErrandVO.setDeliveryTime(errandOrder.getUpdateTime());
        //配送地址
        orderErrandVO.setLocation(orders.getLocation());

        return orderErrandVO;
    }

    @Override
    public List<OrderErrandVO> getListOrderErrandVO(List<Orders> list) {

        return list.stream().map(this::getOrderErrandVO).collect(Collectors.toList());
    }

    @Override
    public List<OrderStatusVO> liststatus(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(Orders::getState, 1,4); //将订单状态为1-已支付和4已出餐的订单拿出来
        queryWrapper.eq(Orders::getUserId, userId);
        List<Orders> ordersList = this.list(queryWrapper);
        List<Long> orderList = ordersList.stream()
                .map(Orders::getId)
                .collect(Collectors.toList());
        //获取哦errandstate
        LambdaQueryWrapper<ErrandOrder> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(ErrandOrder::getOrderId, orderList);
        List<ErrandOrder> errandOrders = errandOrderService.list(wrapper);

        //获取addressimage
        LambdaQueryWrapper<OrderImage> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(OrderImage::getOrderId, orderList);
        List<OrderImage> images = orderImageService.list(lambdaQueryWrapper);
        Map<Long, String> imageMap = images.stream()
                .collect(Collectors.toMap(OrderImage::getOrderId, OrderImage::getImageAddress));
        //先将errandOrder数据变成MAP，提高查询效率
        Map<Long, Integer> map = errandOrders.stream()
                .collect(Collectors.toMap(ErrandOrder::getOrderId, ErrandOrder::getErrandState));
        List<OrderStatusVO> statusVOList = ordersList.stream()
                .map(order -> {
                    OrderStatusVO orderStatusVO = new OrderStatusVO();
                    orderStatusVO.setId(order.getId());
                    orderStatusVO.setOrderState(order.getState());
                    orderStatusVO.setPickupMethod(order.getPickupMethod());
                    orderStatusVO.setImageAddress(imageMap.getOrDefault(order.getId(), ""));
                    orderStatusVO.setErrandState(map.getOrDefault(order.getId(), 0));
                    return orderStatusVO;
                }).collect(Collectors.toList());

        return statusVOList;
    }

    @Override
    public List<OrdersPickVO> getOrdernumber() {
        Map<String, Long> eatIn = this.getQueryNumByDay(0); // 获取堂食单量
        Map<String, Long> self = this.getQueryNumByDay(1); // 获取自提单量
        Map<String, Long> take = this.getQueryNumByDay(2); // 获取外卖单量

        List<OrdersPickVO> ordersPickVOList = new ArrayList<>();

// 遍历 7 天数据（eatIn、self、take 具有相同的日期键）
        for (String date : eatIn.keySet()) {
            OrdersPickVO vo = new OrdersPickVO();
            vo.setDate(date); // 设置日期
            vo.setEatInNum(eatIn.getOrDefault(date, 0L)); // 堂食订单数
            vo.setSelfLiftNum(self.getOrDefault(date, 0L)); // 自提订单数
            vo.setTakeOutNum(take.getOrDefault(date, 0L)); // 外卖订单数
            ordersPickVOList.add(vo);
        }

        return ordersPickVOList;
    }


    private void validateDelivery(String location, LocalDateTime  pickupTime) {
        if (StringUtils.isBlank(location) || !UserConstant.VALID_LOCATIONS.contains(location)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请选择有效地址");
        }
        if (ObjectUtils.isEmpty(pickupTime)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请选择有效取餐时间");
        }
    }

    private BigDecimal calculateTotalPrice(Long orderId, BigDecimal totalPrice, boolean add) {
        LambdaQueryWrapper<OrderDetails> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderDetails::getOrderId, orderId);
        List<OrderDetails> orderDetailsList = orderDetailsService.list(queryWrapper);
        for (OrderDetails orderDetails : orderDetailsList) {
            Long dishesId = orderDetails.getDishesId();
            Dishes dishes = dishesService.getById(dishesId);
            Double packPrice = dishes.getPackPrice();
            BigDecimal pack = BigDecimal.valueOf(packPrice);
            if (add) {
                totalPrice = totalPrice.add(pack);// 确保将结果赋回 totalPrice
            } else {
                totalPrice = totalPrice.subtract(pack); // 减去打包费
            }

        }
//        totalPrice = orderDetailsList.stream()
//                .map(orderDetails -> {
//                    Long dishesId = orderDetails.getDishesId();
//                    Dishes dishes = dishesService.getById(dishesId);
//                    Double packPrice = dishes.getPackPrice();
//                    return BigDecimal.valueOf(packPrice);  // 转换为 BigDecimal
//                })
//                 .reduce(totalPrice, (accumulatedPrice, packPrice) ->
//        add ? accumulatedPrice.add(packPrice) : accumulatedPrice.subtract(packPrice));  // 根据 add 的值进行加减
        return totalPrice;
    }

    private Business getBusiness (Long businessId) {
        Business business = null;
        if (businessId != null && businessId > 0) {
            business = businessService.getById(businessId);
        }
        return business;
    }

    private Map<String, Long> getQueryNumByDay(int pickupMethod) {
        Map<String, Long> resultMap = new LinkedHashMap<>();
        LocalDateTime now = LocalDateTime.now();

        for (int i = 0; i < 7; i++) {
            LocalDateTime startOfDay = now.minusDays(i).toLocalDate().atStartOfDay();
            LocalDateTime endOfDay = startOfDay.plusDays(1);

            LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.ge(Orders::getCreateTime, startOfDay);
            queryWrapper.lt(Orders::getCreateTime, endOfDay);
            queryWrapper.eq(Orders::getPickupMethod, pickupMethod);

            long count = this.count(queryWrapper);
            resultMap.put(startOfDay.toLocalDate().toString(), count);
        }

        return resultMap;
    }

    private BigDecimal calculateTotal(Long orderId, BigDecimal totalPrice, String location, boolean add) {
        Orders orders = this.getById(orderId);
        log.info("location:{}" , location);
        BigDecimal pack = DELIVERY_FEE_MAP.get(location);
        log.info("pack:{}" , pack);
        if (add) {
            totalPrice = totalPrice.add(pack);// 确保将结果赋回 totalPrice
        } else {
            totalPrice = totalPrice.subtract(pack); // 减去打包费
        }

        return totalPrice;
    }

}